@@ -5,6 +5,7 @@
 #include "messageviewerdialog.h"
 
 #include "messageviewer.h"
+#include "messageviewerutils_p.h"
 #include <MimeTreeParserCore/CryptoHelper>
 #include <MimeTreeParserCore/FileOpener>
 
@@ -29,56 +30,6 @@
 
 using namespace MimeTreeParser::Widgets;
 
-namespace
-{
-
-inline QString changeExtension(const QString &fileName, const QString &extension)
-{
-    auto renamedFileName = fileName;
-    renamedFileName.replace(QRegularExpression(QStringLiteral("\\.(mbox|p7m|asc)$")), extension);
-
-    // In case the file name didn't contain any of the expected extension: mbox, p7m, asc and eml
-    // or doesn't contains an extension at all.
-    if (!renamedFileName.endsWith(extension)) {
-        renamedFileName += extension;
-    }
-
-    return renamedFileName;
-}
-
-#define SLASHES "/\\"
-
-inline QString changeFileName(const QString &fileName, const QString &subject)
-{
-    if (subject.isEmpty()) {
-        return fileName;
-    }
-
-    if (fileName.isEmpty()) {
-        return subject;
-    }
-
-    auto cleanedSubject = subject;
-
-    static const char notAllowedChars[] = ",^@={}[]~!?:&*\"|#%<>$\"'();`'/\\.";
-    static const char *notAllowedSubStrings[] = {"..", "  "};
-
-    for (const char *c = notAllowedChars; *c; c++) {
-        cleanedSubject.replace(QLatin1Char(*c), QStringLiteral(" "));
-    }
-
-    const int notAllowedSubStringCount = sizeof(notAllowedSubStrings) / sizeof(const char *);
-    for (int s = 0; s < notAllowedSubStringCount; s++) {
-        const QLatin1StringView notAllowedSubString(notAllowedSubStrings[s]);
-        cleanedSubject.replace(notAllowedSubString, QStringLiteral(" "));
-    }
-
-    QStringList splitedFileName = fileName.split(QLatin1Char('/'));
-    splitedFileName[splitedFileName.count() - 1] = cleanedSubject;
-    return splitedFileName.join(QLatin1Char('/'));
-}
-}
-
 class MessageViewerDialog::Private
 {
 public:
@@ -177,19 +128,20 @@ void MessageViewerDialog::Private::save(QWidget *parent)
     if (wasEncrypted) {
         extension = QStringLiteral(".mime");
         if (protocol == GpgME::OpenPGP) {
-            alternatives = i18nc("File dialog accepted files", "Email files (*.eml *.mbox *.mime)");
+            alternatives = i18nc("File dialog accepted files", "EML file (*.eml);;MBOX file (*.mbox);;MIME file (*.mime)");
         } else {
             alternatives = i18nc("File dialog accepted files", "Encrypted S/MIME files (*.p7m)");
         }
     } else {
         extension = QStringLiteral(".eml");
-        alternatives = i18nc("File dialog accepted files", "Email files (*.eml *.mbox *.mime)");
+        alternatives = i18nc("File dialog accepted files", "EML file (*.eml);;MBOX file (*.mbox);;MIME file (*.mime)");
     }
 
-    const QString location = QFileDialog::getSaveFileName(parent,
-                                                          i18nc("@title:window", "Save File"),
-                                                          changeExtension(changeFileName(fileName, messageViewer->subject()), extension),
-                                                          alternatives);
+    const QString location =
+        QFileDialog::getSaveFileName(parent,
+                                     i18nc("@title:window", "Save File"),
+                                     MesageViewerUtils::changeExtension(MesageViewerUtils::changeFileName(fileName, messageViewer->subject()), extension),
+                                     alternatives);
 
     QSaveFile file(location);
     if (!file.open(QIODevice::WriteOnly)) {
@@ -202,10 +154,11 @@ void MessageViewerDialog::Private::save(QWidget *parent)
 
 void MessageViewerDialog::Private::saveDecrypted(QWidget *parent)
 {
-    const QString location = QFileDialog::getSaveFileName(parent,
-                                                          i18nc("@title:window", "Save Decrypted File"),
-                                                          changeExtension(changeFileName(fileName, messageViewer->subject()), QStringLiteral(".eml")),
-                                                          i18nc("File dialog accepted files", "Email files (*.eml *.mbox *.mime)"));
+    const QString location = QFileDialog::getSaveFileName(
+        parent,
+        i18nc("@title:window", "Save Decrypted File"),
+        MesageViewerUtils::changeExtension(MesageViewerUtils::changeFileName(fileName, messageViewer->subject()), QStringLiteral(".eml")),
+        i18nc("File dialog accepted files", "EML file (*.eml);;MBOX file (*.mbox);;MIME file (*.mime)"));
 
     QSaveFile file(location);
     if (!file.open(QIODevice::WriteOnly)) {
